% HSTVOLREADER.M
%
% Usage: vol=HSTVolReader(filename{,'xrange',xrange,'yrange',yrange,'zrange',zrange})
%        where filename can be any of the following: vname.vol,
%        vname.vol.xml, or just vname.
%        The three optional ranges are in the usual form for
%        Matlab - 1:10 will do the first 10 elements,
%        20:5:50 will do every 5th element from 20 to 50 inclusive.  If the
%        optional ranges are not specified, the user will be told the size
%        of the volume, then asked what ranges s/he would like to read.
%
% Examples:
%      vol=HSTVolReader('myvolume','xrange',200:300,'yrange',400:5:500,'zrange'1:10)
%          reads from myvolume.vol the every voxel in X between 200 and
%          300, every 5th voxel in Y between 400 and 500, and the first 10
%          slices in Z.
%
%      vol=HSTVolReader('yourvolume','xrange','all','zrange',50:60,'yrange','all');
%          reads from yourvolume.vol the complete Z slices between 50 and
%          60.  NB that the parameters can be in ANY order
%
%
% NB:  Reading 32 bit files (.vol) gives output as 'single's - 32 bit
%      floats. Reading 8 bit files (.raw) gives output as 'uint8's - 8 bit
%      unsigned integers.  These take up much less space, but not all
%      calculations can be done with them.  If you want them as floating point
%      numbers, use:
%
%        vol8=HSTVolReader(my8bitfile...);
%        vol32=single(vol8);   %  for 32 bit variable OR
%        vol64=double(vol8);   % four 64 bit (double) variable - the matlab
%                                default type.

% PS:  If you copy this function, look at the internal comments (too long for the help) to 
%      know what else you need.
%        
%
% Greg Johnson, February 2005
% greg.johnson@esrf.fr
%
% Modified December 2006
% Now works with 8 bit files also (.raw)!  Outputs uint8 variables
%
% Modified January 2007
% Now does ring correction!  Documentation needs completion but here is an
% example:
%
% HSTVolReader('/data/id19/external/in537/volfloat/a10_0001','functionhandle',@remove_rings,'functionarguments',[13.8])
%
% This reads the volume and calls the ring removal function with a
% threshold value (to segment sample from background) of 13.8.  Each
% corrected slice is written as an EDF file to the CURRENT directory.  This
% shall be condored soon to permit much faster work
% The easier function to call is
%
% sequence_remove_rings('/data/id19/myvolume',13.8)
% 
% To guess the threshold, load a single slice from the volume (see above)
% and run
%
% autothresh(im)
% 
% Have a look at the ID19 website (Software section) on http://www.esrf.eu


% Required files: you need (not all, but easiest just to take the entire directories
%        matlab/moretomo/
%        matlab/utils/
%
% Individual files:
%        jdom.jar
%        volread_single.m
%        parse_pv_pairs.m
%        HST_info.m 
%        inputwdefaultnumeric.m
%        edf_write.m 


function varargout=HSTVolReader(varargin)


if mod(nargin,2)==1
  %  disp('Odd parameters...');
  full_fname=varargin{1};
  varargin=varargin(2:end);
end
params.xrange='all';
params.yrange='all';
params.zrange='all';
params.functionhandle='';
params.functionarguments=[];
params.outputfile='';

params=parse_pv_pairs(params,varargin);

if ~exist('full_fname','var')  % filename not specified on command line
  [tmp1,tmp2]=uigetfile('*','Select a .vol or .raw volume file');
  full_fname=fullfile(tmp2,tmp1);
end

try
  HST_header=HST_info(full_fname);
catch
  rethrow(lasterror)
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%
% common for both filetypes

if nargin <= 1 % no parameters except possibly the filename
  if HST_header.volSizeZ==1 % if there is only one slice, don't bother asking the user and just read the whole thing
    rangeX=1:HST_header.volSizeX;
    rangeY=1:HST_header.volSizeY;
    rangeZ=1:HST_header.volSizeZ;
  else
    disp('Interactive choice of ROI')
    % interactively choose what to read
    disp(sprintf('Volume size: X %d Y %d Z %d',HST_header.volSizeX,HST_header.volSizeY,HST_header.volSizeZ));
    disp(sprintf('Specify ROI to read'));
    str=sprintf('1:%d',HST_header.volSizeX);
    rangeX=inputwdefaultnumeric('X',str);
    str=sprintf('1:%d',HST_header.volSizeY);
    rangeY=inputwdefaultnumeric('Y',str);
    str=sprintf('1:%d',HST_header.volSizeZ);
    rangeZ=inputwdefaultnumeric('Z',str);
  end
else
  % user has specified x y and z ranges
  % might have used 'all' to indicate all elements
  if strcmp(params.xrange,'all')
    rangeX=1:HST_header.volSizeX;
  else
    rangeX=params.xrange;
  end
  if strcmp(params.yrange,'all')
    rangeY=1:HST_header.volSizeY;
  else
    rangeY=params.yrange;
  end
  if strcmp(params.zrange,'all')
    rangeZ=1:HST_header.volSizeZ;
  else
    rangeZ=params.zrange;
  end
end

skip_voxels=(HST_header.volSizeX)*(HST_header.volSizeY)*(rangeZ(1)-1);
skip_bytes=skip_voxels*HST_header.bytespervoxel;

if nargout==1
  % this is not very efficient for non-contiguous slices.  One day, this
  % should be improved...
  % *** should include memory mapped code from volviewer - useful
  % for larger volumes
  if ~exist(HST_header.fname_final,'file')
    error(sprintf('File (%s) does not exist!',HST_header.fname_final))
    return
  end

% if reading an 8 bit file, give the output as 8 bits, too.  Saves A LOT of
% memory...
  if strcmpi(HST_header.ftype,'uint8')
    customftype='uint8=>uint8';
  else
    customftype=HST_header.ftype;
  end

  vol=volread_single(HST_header.fname_final,customftype,skip_bytes,HST_header.volSizeX,HST_header.volSizeY,rangeZ(end)-rangeZ(1)+1,HST_header.fbyteorder);
  
  vol=vol(rangeX,rangeY,rangeZ-rangeZ(1)+1);
  varargout{1}=vol;
  if nargin<4
    disp('You could also run this command directly - see help for details:')
  end
else % TODO put argument check at beginning of function

  if ~exist(HST_header.fname_final,'file')
    error(sprintf('File (%s) does not exist!',HST_header.fname_final))
    return
  end
  if ~isa(params.functionhandle,'function_handle')
    disp('I need a valid functionhandle!')
    return
  else
    % read slice by slice, process, and save
    for slicenum=1:(rangeZ(end)-rangeZ(1)+1)
      
      fprintf('Working on slice %d/%d...\n',rangeZ(slicenum),rangeZ(end));
      % always read as singles, even if 8 bit - calculations might need the
      % precision...
      
      skip_voxels=(HST_header.volSizeX)*(HST_header.volSizeY)*(rangeZ(slicenum)-1);
      skip_bytes=skip_voxels*HST_header.bytespervoxel;

      im=volread_single(...
        HST_header.fname_final,...
        HST_header.ftype,...
        skip_bytes,...
        HST_header.volSizeX,...
        HST_header.volSizeY,...
        1,...
        HST_header.fbyteorder);
      % process slice
      if ~isempty(params.functionarguments)
        im2=params.functionhandle(im,params.functionarguments);
      else
        im2=params.functionhandle(im);
      end
      % save slice
      outputname=sprintf('%s_RC_%04d.edf',HST_header.fname_prefix,rangeZ(slicenum));
      edf_write(single(im2),outputname);
    end
    %    vol=vol(rangeX,rangeY,rangeZ-rangeZ(1)+1);
  end

end

