% VOL_VIEW.M  A simple volume viewer
% Usage:
% vol_view(vol)  Will show the simplest version of the viewer.  Try it!
% vol_view(vol,'parameter','value') - specify some optional parameters.
% These are:
% 'plane'  =  'xy','yz','zx'  << viewing plane
% 'slicendx'= 42              << start viewing at slice index supplied
% 'cmap'   =   gray,jet,etc   << any colour map. See 'help graph3d'
% 'cbar'   =   true,false     << show colourbar
% 'clims'  =   2, [min max]   << either a single value (percentile) or two
%                                absolute values
% 'ij'     =   true,false     << use ImageJ for viewing
%
% Examples:
% vol_view(vol,'cbar',true,'plane','yz','slicendx',43,'cmap',jet,'clims',3)
%
% You can use vol=HSTVolReader to load your ESRF HST volumes.


function vol_view(vol,varargin)
app=[];
firsttime=true;
sfInitialise(varargin)
if app.ij
  % use ImageJ instead of Matlab figure
  ij_imshow(vol)
  disp('ImageJ version does not support plane,cmap, or cbar yet');
else
  sfSetupFigure
  sfUpdateDisplay
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  function sfInitialise(arguments)
    warning('off','Images:initSize:adjustingMag');
    warning('off','MATLAB:divideByZero')

    app.slicendx=1;
    app.clims=2;
    app.cbar=false;
    app.cmap=gray;
    app.plane='xy';
    app.ij=false;
    if ~isempty(arguments)
      % if the user passed in some parameters, use them
      app=parse_pv_pairs(app,arguments);
    end

    % now validate the arguments
    if ~strcmp(app.plane,{'xy','yz','zx'})
      disp('Plane must be one of ''xy'',''yz'',''zx''')
      help mfilename
      return
    end
    % ensure slice is possible for a given orientation
    switch app.plane
      case 'xy'
        tmpdim=3;
      case 'yz'
        tmpdim=1;
      case 'zx'
        tmpdim=2;
    end
    slicendx=min(app.slicendx,size(vol,tmpdim));
  end




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  function sfSetupFigure
    zoom off
    app.h_slider=uicontrol('style','slider');
    set(app.h_slider,'callback',@sfUpdateDisplay)
    set(app.h_slider,'units','normalized');
    set(app.h_slider,'position',[0 0 1 0.05])
    switch app.plane
      case 'xy'
        set(app.h_slider,'min',1,'max',size(vol,3),'sliderstep',[1/size(vol,3) 1/size(vol,3)]);
      case 'yz'
        set(app.h_slider,'min',1,'max',size(vol,1),'sliderstep',[1/size(vol,1) 1/size(vol,1)]);
      case 'zx'
        set(app.h_slider,'min',1,'max',size(vol,2),'sliderstep',[1/size(vol,2) 1/size(vol,1)]);
    end

    set(app.h_slider,'value',app.slicendx);
    set(app.h_slider,'tag','slider');
    if size(vol,3)==1
      set(app.h_slider,'visible','off')
    end
    set(gcf,'windowbuttonmotionfcn',@sfUpdateDisplay)
    set(gcf,'windowbuttondownfcn',@sfUpdateDisplay)
    set(gcf,'currentobject',app.h_slider); % pretned the user has already clicked on the slider

    set(gcf,'menuBar','none')

    % modify the colour limits if a percentile is requested
    app.clims=pfSetColourLimits(vol,app.clims,'verbose',true);

    zoom on
  end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
  function sfUpdateDisplay(varargin)
    if (isequal(gco,app.h_slider) || firsttime)
      tmp=round(get(app.h_slider,'value'));
      set(app.h_slider,'value',tmp);
      if (tmp~=app.slicendx | firsttime)
        app.slicendx=tmp;
        set(app.h_slider,'value',app.slicendx);
        if firsttime
          switch app.plane
            case 'xy'
              slice=vol(:,:,1);
            case 'yz'
              slice=squeeze(vol(1,:,:));
            case 'zx'
              slice=squeeze(vol(:,1,:));
          end
          xtmp=[0.5 size(slice,2)+0.5];
          ytmp=[0.5 size(slice,1)+0.5];
        else
          xtmp=get(gca,'xlim');
          ytmp=get(gca,'ylim');
        end
        switch app.plane
          case 'xy'
            imshow(vol(:,:,round(app.slicendx)),app.clims)
          case 'yz'
            imshow(squeeze(vol(round(app.slicendx),:,:)),app.clims)
          case 'zx'
            imshow(squeeze(vol(:,round(app.slicendx),:)),app.clims)
        end
        %        axis image
        axis on
        switch(app.plane)
          case 'xy'
            xlabel('X');
            ylabel('Y');
          case 'yz'
            xlabel('Y');
            ylabel('Z');
          case 'zx'
            xlabel('Z');
            ylabel('X');
        end
        set(gca,'xlim',xtmp)
        set(gca,'ylim',ytmp)
        colormap(app.cmap)
        if app.cbar
          colorbar
        end
        title(['Slice: ', num2str(app.slicendx)])
        firsttime=false;
        shg
      end
    end
  end

end
